@@ -16,8 +16,98 @@ DESCRIPTION = """
 
 import random
 from utils import *
-from proto.dataset_pb2 import Sim2Query, Search, Dataset
 from google.protobuf import text_format
+import cPickle as pickle
+import json
+
+
+class Dataset(object):
+    """ This class holds the information of a dataset.
+
+    Layout of dataset: refer the `validate` function
+    """
+
+    def __init__(self, name=None, desc=None, images=None, labels=None,
+                 dblst=None, qrylst=None, v_simlst=None,
+                 trlst=None, valst=None):
+        # name of dataset
+        self.name = name
+        # description to this dataset
+        self.desc = desc
+        # list of image paths
+        self.images = images
+        # list of image labels
+        self.labels = labels
+
+        # list of database images
+        self.dblst = dblst
+        # list of query images
+        self.qrylst = qrylst
+        # list of similairties [required if labels is not specified]
+        self.v_simlst = v_simlst
+
+        # (optional) list of training images for DML methods
+        self.trlst = trlst
+        # (optional) list of validation images for DML methods
+        self.valst = valst
+
+    @staticmethod
+    def is_list_of(elem_list, elem_type):
+        """
+        Check if `elem_list` is a list of `elem_type`.
+        """
+        if type(elem_list) is not list:
+            return False
+
+        for elem in elem_list:
+            if type(elem) is not elem_type:
+                return False
+        return True
+
+
+    def validate(self):
+        """
+        Validate each fields
+        """
+
+        msg = "`name` and `desc` should be `str`"
+        if type(self.name) is not str or type(self.desc) is not str:
+            raise Exception(msg)
+
+        msg = "`images` should be a list of `str`"
+        if not Dataset.is_list_of(self.images, str):
+            raise Exception(msg)
+
+        msg = "`labels` should be a list of int"
+        if not Dataset.is_list_of(self.labels, int):
+            raise Exception(msg)
+
+        msg = "`dblst` should be a list of int"
+        if not Dataset.is_list_of(self.dblst, int):
+            raise Exception(msg)
+
+        msg = "`qrylst` should be a list of int"
+        if not Dataset.is_list_of(self.qrylst, int):
+            raise Exception(msg)
+
+        msg = "`v_simlst` should be `dict`"
+        msg_key = "`key` in v_simlst should be `float`"
+        msg_val = "`val` in v_simlst should be a list of `int`"
+        if type(self.v_simlst) is not dict:
+            raise Exception(msg)
+        for key, val in self.v_simlst.iteritems():
+            if type(key) is not float:
+                raise Exception(msg_key)
+            if Dataset.is_list_of(val, int):
+                raise Exception(msg_val)
+
+        msg = "`trlst` should be a list of int"
+        if self.trlst and not Dataset.is_list_of(self.trlst, int):
+            raise Exception(msg)
+
+        msg = "`valst` should be a list of int"
+        if self.valst and not Dataset.is_list_of(self.valst, int):
+            raise Exception(msg)
 
 
 class Builder(object):
@@ -37,11 +127,11 @@ class Builder(object):
     POS_LIST = 'search/pos.lst'
     AMB_LIST = 'search/amb.lst'
     TRAIN_LIST = 'search/train.lst'
+    VAL_LIST = 'search/val.lst'
 
     def __init__(self):
         # protobuf
-        self.dataset = Dataset()
-        self.search = Search()
+        self.ds = Dataset()
 
         # path or name of images
         self.imglst = None
@@ -59,6 +149,8 @@ class Builder(object):
         self.amblst = None
         # images for training
         self.trainlst = None
+        # images for training
+        self.val_lst = None
 
         # print the doc string, which includes the PRE-REQURIES of
         # the source dir
@@ -80,56 +172,10 @@ class Builder(object):
         """
         return ".".join(os.path.split(filepath)[1].split('.')[:-1])
 
-    @staticmethod
-    def ilst(arr):
-        """
-        Convert an array to int32 list
-        """
-        return [int(item) for item in arr]
-
-    def generate_protobuf(self):
-        """
-        Packing and generate protobuf file
-        """
-        pinfo("Packing dataset protobuf ...\n")
-        self.dataset.name = "Hi"
-        self.dataset.comment = "Bye"
-        self.dataset.images.extend(self.imglst)
-        print self.dataset.labels
-        print type(list(self.lbslst))
-        print type(list(self.lbslst)[0])
-        self.dataset.labels.extend(self.ilst(self.lbslst))
-        pinfo("Done!\n")
-
-        pinfo("Packing search protobuf ...\n")
-        self.search.dblst.extend(self.ilst(self.dblst))
-        self.search.trainlst.extend(self.ilst(self.trainlst))
-
-        for qid in range(len(self.qrylst)):
-            query = self.search.qrylst.add()
-            query.query = self.qrylst[qid]
-            query.poslst.extend(self.ilst(self.poslst[qid]))
-            query.amblst.extend(self.ilst(self.amblst[qid]))
-
-            for val, lsts in self.s2qlst.iteritems():
-                sim2qrylst = query.sim2qrylst.add()
-                sim2qrylst.sim2qry = val
-                sim2qrylst.imglst.extend(self.ilst(lsts[qid]))
-        pinfo("Done!\n")
-
-
     def generate(self, trgdir):
         """ Generate the dataset related files.
         """
 
-        self.generate_protobuf()
-
-        with open("%s/dataset.proto" % trgdir, "w") as dsf:
-            dsf.write(text_format.MessageToString(self.dataset))
-
-        with open("%s/search.proto" % trgdir, "w") as shf:
-            shf.write(text_format.MessageToString(self.search))
-
         pinfo("Generating dataset files ")
         # create target dir
         os.system("mkdir -p %s/search" % trgdir)
@@ -161,6 +207,12 @@ class Builder(object):
             np.savetxt(os.sep.join([trgdir, Builder.TRAIN_LIST+"_1"]),
                        [iid+1 for iid in self.trainlst], '%s')
             pinfo(".")
+        if None != self.val_lst:
+            np.savetxt(os.sep.join([trgdir, Builder.VAL_LIST]),
+                       self.val_lst, '%s')
+            np.savetxt(os.sep.join([trgdir, Builder.VAL_LIST+"_1"]),
+                       [iid+1 for iid in self.val_lst], '%s')
+            pinfo(".")
         if None != self.s2qlst:
             for val, lsts in self.s2qlst.iteritems():
                 save_lst_of_lst(os.sep.join([trgdir, Builder.S2Q_LIST +
@@ -182,75 +234,120 @@ class Builder(object):
             pinfo(".")
         pinfo(" Done!\n")
 
-
-class Oxford(Builder):
-    """ Oxford-dataset builder.
-    Download: http://www.robots.ox.ac.uk/~vgg/data/oxbuildings/
-
-    !!NOTE!! `all_souls_000183.jpg` appeares in both `all_souls` and
-             `radcliffe_camera` classes
-    """
-
-    ALL_LANDMARKS = ['all_souls', 'ashmolean', 'balliol', 'bodleian',
-                     'christ_church', 'cornmarket', 'hertford', 'keble',
-                     'magdalen', 'pitt_rivers', 'radcliffe_camera']
-
-    def __init__(self, args):
-        """ Parse the Oxford dataset
+    def save(self, path):
         """
+        Save the dataset to pkl file.
+        """
+        self.ds.name = "name"
+        self.ds.desc = "desc"
+        self.ds.images = list(self.imglst)
+        self.ds.labels = [int(lbl) for lbl in self.lbslst]
+
+        self.ds.dblst = [int(iid) for iid in self.dblst]
+        self.ds.qrylst = [int(iid) for iid in self.qrylst]
+        self.ds.v_simlst = self.s2qlst
+
+        self.ds.trlst = [int(iid) for iid in self.trainlst]
+        self.ds.valst = [int(iid) for iid in self.val_lst]
+
+        self.ds.validate()
+        with open(path, 'wb') as pklf:
+            pickle.dump(self.ds, pklf)
+
+    def load(self, path):
+        """
+        Load the dataset from pkl file.
+        """
+        with open(path, 'rb') as pklf:
+            self.ds = pickle.load(pklf)
+        self.ds.validate()
+
+    def savetxt(self, path):
+        """
+        Save the dataset to text file.
+        """
+        raise Exception("Not implemented yet!")
+        self.ds.validate()
+
+    def loadtxt(self, path):
+        """
+        Load the dataset from text file.
+        """
+        raise Exception("Not implemented yet!")
+        self.ds.validate()
+
+
+class VGGBuilder(Builder):
+    """ VGG dataset builder.
+    This is the base class for `Oxford` and `Paris`.
+    """
+    def __init__(self, args):
         Builder.__init__(self)
 
         pinfo("Loading image list ... ")
+        self.imglst = []
         # convert the image path into relative path to the target dir
-        imnames = [item.split(".jpg")[0] for item in
-                   sorted(os.listdir(args.img_dir))]
-        self.imglst = [os.path.relpath(os.sep.join([args.img_dir,
-                                                    imname + ".jpg"]),
-                                       args.trg_dir) for imname in imnames]
-        self.lbslst = np.zeros(len(self.imglst), np.int32)
+        for dpath, _, files in os.walk(args.img_dir):
+            for fpath in files:
+                self.imglst.append(os.path.relpath(os.path.join(dpath, fpath),
+                                                   args.trg_dir))
+        self.imglst = sorted(self.imglst)
+        imnames = [Builder.get_name(path) for path in self.imglst]
+        self.lbslst = np.zeros(len(self.imglst), np.int)
         pinfo("Done!\n")
 
         self.qrylst = []
         self.poslst = []
         self.amblst = []
         self.s2qlst = {args.good: [], args.ok: [], args.junk: []}
+        self.trainlst = []
+        self.val_lst = []
 
-        clsnum = len(Oxford.ALL_LANDMARKS)
+        clsnum = len(self.ALL_LANDMARKS)
         # class id start from 1, 0 denotes background images
-        for (clsid, landmark) in zip(range(1, clsnum + 1),
-                                     Oxford.ALL_LANDMARKS):
+        for (cls_id, landmark) in zip(range(1, clsnum + 1),
+                                     self.ALL_LANDMARKS):
             pinfo("\t%s\n" % landmark)
             for i in range(5):
                 gtf = "%s_%d" % (landmark, i+1)
 
+                # get query Id
                 qname = np.loadtxt("%s/%s_query.txt" % (args.gt_dir, gtf), dtype=np.str).\
                     tolist()[0].replace('oxc1_', '')
                 qid = imnames.index(qname)
                 self.qrylst.append(qid)
 
+                # get good results
                 good_names = loadlist("%s/%s_good.txt" % (args.gt_dir, gtf))
                 good_ids = [imnames.index(x) for x in good_names]
                 self.s2qlst[args.good].append(good_ids)
 
+                # get ok results
                 ok_names = loadlist("%s/%s_ok.txt" % (args.gt_dir, gtf))
                 ok_ids = [imnames.index(x) for x in ok_names]
                 self.s2qlst[args.ok].append(ok_ids)
 
-                cur_poslst = np.array(ok_ids + good_ids)
-                # cur_poslst = cur_poslst[cur_poslst != qid]
-                self.poslst.append(list(cur_poslst))
-
+                # get junk results
                 junk_names = loadlist("%s/%s_junk.txt" % (args.gt_dir, gtf))
                 junk_ids = [imnames.index(x) for x in junk_names]
                 self.s2qlst[args.junk].append(junk_ids)
-                self.amblst.append(junk_ids)
-            self.lbslst[self.poslst[-1]] = clsid
 
+                cur_poslst = np.array(ok_ids + good_ids)
+                # DONOT exclude query from the positive list
+                # cur_poslst = cur_poslst[cur_poslst != qid]
+                self.poslst.append(list(cur_poslst))
+                self.amblst.append(junk_ids)
+
+            # set labels
+            self.lbslst[self.poslst[-1]] = cls_id
+
+        # set db list
         self.dblst = np.array(range(len(self.imglst)))
 
         self.trainlst = list(set(range(len(self.imglst))).difference(set(self.qrylst)))
+
         # self.trainlst = []
-        # for classid in range(len(Oxford.ALL_LANDMARKS)):
+        # for classid in range(len(self.ALL_LANDMARKS)):
         #     # select images with the same IDs, BUT NOT IN THE QUERY LIST
         #     candidates = [iid for iid in self.dblst[self.lbslst == (classid + 1)]
         #                   if iid not in self.qrylst]
@@ -263,213 +360,17 @@ class Oxford(Builder):
 
         pinfo("\n")
 
-    def __del__(self):
-        pass
-
-    @classmethod
-    def set_parser(cls, parser):
-        Builder.set_parser(parser)
-        parser.add_argument('img_dir', type=str,
-                            help="""dir containing images extracted from
-                            `oxbuild_images.tgz`""")
-        parser.add_argument('gt_dir', type=str,
-                            help="""dir containing groundtruth extracted from
-                            `gt_files_170407.tgz`""")
-        parser.add_argument('--good', type=float, default=3.0,
-                            help="gain value for `good` results")
-        parser.add_argument('--ok', type=float, default=2.0,
-                            help="gain value for `ok` results")
-        parser.add_argument('--junk', type=float, default=1.0,
-                            help="gain value for `junk` results")
-        parser.add_argument('--selpos', type=int, default=7,
-                            help="NO. of positive image for each class")
-        parser.add_argument('--selneg', type=int, default=500,
-                            help="number of selected negative")
-
-
-class OxfordM(Oxford):
-    """ Metric Learning Edition of `Oxford`
-    """
-
-    # selected classes for metric learning
-    SEL_LANDMARKS = ['all_souls', 'ashmolean', 'bodleian', 'christ_church',
-                     'hertford', 'magdalen', 'radcliffe_camera']
-    # SEL_LANDMARKS = Oxford.ALL_LANDMARKS
-    #
-
-    def __init__(self, args):
-        Oxford.__init__(self, args)
-
-        all_classes = ['background'] + Oxford.ALL_LANDMARKS
-
-        pinfo("Preparing for metric learning ... ")
-        self.trainlst = []
-        qry_lbs = self.lbslst[self.qrylst]
-        qry_mask = np.zeros(len(self.qrylst))
-        for sel_landmark in OxfordM.SEL_LANDMARKS:
-            classid = all_classes.index(sel_landmark)
-            qry_mask[qry_lbs == classid] = 1
-            # select images with the same IDs, BUT NOT IN THE QUERY LIST
-            candidates = [iid for iid in self.dblst[self.lbslst == (classid+1)]
-                          if iid not in self.qrylst]
-            random.shuffle(candidates)
-            self.trainlst += candidates[:args.selpos]
-
-        candidates = list(self.dblst[self.lbslst == 0])
-        random.shuffle(candidates)
-        self.trainlst += candidates[:500]
-
-        self.qrylst = [self.qrylst[qid] for qid in range(len(self.qrylst))
-                       if qry_mask[qid] > 0]
-        self.poslst = [self.poslst[qid] for qid in range(len(self.poslst))
-                       if qry_mask[qid] > 0]
-        self.amblst = [self.amblst[qid] for qid in range(len(self.amblst))
-                       if qry_mask[qid] > 0]
-
-        self.dblst = [iid for iid in self.dblst
-                      if iid not in self.trainlst]
-        for i in range(len(self.qrylst)):
-            self.poslst[i] = [iid for iid in self.poslst[i]
-                              if iid not in self.trainlst]
-            self.amblst[i] = [iid for iid in self.amblst[i]
-                              if iid not in self.trainlst]
-        for key in self.s2qlst.keys():
-            self.s2qlst[key] = [self.s2qlst[key][qid] for qid
-                                in range(len(self.s2qlst[key]))
-                                if qry_mask[qid] > 0]
-            for i in range(len(self.qrylst)):
-                self.s2qlst[key][i] = [iid for iid in self.s2qlst[key][i]
-                                       if iid not in self.trainlst]
-        self.dblst = [iid for iid in self.dblst
-                      if iid not in self.trainlst]
-        pinfo("Done!\n")
-
-    @classmethod
-    def set_parser(cls, parser):
-        Oxford.set_parser(parser)
-
-
-class Paris(Builder):
-    """ Paris-dataset builder.
-    Download: http://www.robots.ox.ac.uk/~vgg/data/parisbuildings/
-    """
-
-    ALL_LANDMARKS = ['defense', 'eiffel', 'invalides', 'louvre', 'moulinrouge',
-                     'museedorsay', 'notredame', 'pantheon', 'pompidou',
-                     'sacrecoeur', 'triomphe']
-
-    def __init__(self, args):
-        """ Parse the Parse dataset
+    def init_M(self):
         """
-        Builder.__init__(self)
-
-        pinfo("Loading image list ... ")
-        self.imglst = []
-        # convert the image path into relative path to the target dir
-        for dpath, _, files in os.walk(args.img_dir):
-            for fpath in files:
-                self.imglst.append(os.path.relpath(os.path.join(dpath, fpath),
-                                                   args.trg_dir))
-        self.imglst = sorted(self.imglst)
-        imnames = [Builder.get_name(path) for path in self.imglst]
-        self.lbslst = np.zeros(len(self.imglst), np.int32)
-        pinfo("Done!\n")
-
-        self.qrylst = []
-        self.poslst = []
-        self.amblst = []
-        self.s2qlst = {args.good: [], args.ok: [], args.junk: []}
-
-        clsnum = len(Oxford.ALL_LANDMARKS)
-        # class id start from 1, 0 denotes background images
-        for (clsid, landmark) in zip(range(1, clsnum + 1),
-                                     Paris.ALL_LANDMARKS):
-            pinfo("\t%s\n" % landmark)
-            for i in range(5):
-                gtf = "%s_%d" % (landmark, i+1)
-
-                qname = np.loadtxt("%s/%s_query.txt" % (args.gt_dir, gtf), dtype=np.str).\
-                    tolist()[0].replace('oxc1_', '')
-                qid = imnames.index(qname)
-                self.qrylst.append(qid)
-
-                good_names = loadlist("%s/%s_good.txt" % (args.gt_dir, gtf))
-                good_ids = [imnames.index(x) for x in good_names]
-                self.s2qlst[args.good].append(good_ids)
-
-                ok_names = loadlist("%s/%s_ok.txt" % (args.gt_dir, gtf))
-                ok_ids = [imnames.index(x) for x in ok_names]
-                self.s2qlst[args.ok].append(ok_ids)
-
-                cur_poslst = np.array(ok_ids + good_ids)
-                # cur_poslst = cur_poslst[cur_poslst != qid]
-                self.poslst.append(list(cur_poslst))
-
-                junk_names = loadlist("%s/%s_junk.txt" % (args.gt_dir, gtf))
-                junk_ids = [imnames.index(x) for x in junk_names]
-                self.s2qlst[args.junk].append(junk_ids)
-                self.amblst.append(junk_ids)
-            self.lbslst[self.poslst[-1]] = clsid
-            print clsid, len(self.poslst[-1])
-
-        self.dblst = np.array(range(len(self.imglst)))
-
-        self.trainlst = list(set(range(len(self.imglst))).difference(set(self.qrylst)))
-        # self.trainlst = []
-        # for classid in range(len(Paris.ALL_LANDMARKS)):
-        #     # select images with the same IDs, BUT NOT IN THE QUERY LIST
-        #     candidates = [iid for iid in self.dblst[self.lbslst == (classid+1)]
-        #                   if iid not in self.qrylst]
-        #     random.shuffle(candidates)
-        #     self.trainlst += candidates[:args.selpos]
-
-        # candidates = list(self.dblst[self.lbslst == 0])
-        # random.shuffle(candidates)
-        # self.trainlst += candidates[:args.selneg]
-
-        pinfo("\n")
-
-    def __del__(self):
-        pass
-
-    @classmethod
-    def set_parser(cls, parser):
-        Builder.set_parser(parser)
-        parser.add_argument('img_dir', type=str,
-                            help="""dir containing images extracted from
-                            `paris_1.tgzk` and `paris_2.tgz`""")
-        parser.add_argument('gt_dir', type=str,
-                            help="""dir containing groundtruth extracted from
-                            `paris_120310.tgz`""")
-        parser.add_argument('--good', type=float, default=3.0,
-                            help="gain value for `good` results")
-        parser.add_argument('--ok', type=float, default=2.0,
-                            help="gain value for `ok` results")
-        parser.add_argument('--junk', type=float, default=1.0,
-                            help="gain value for `junk` results")
-        parser.add_argument('--selpos', type=int, default=15,
-                            help="NO. of positive image for each class")
-        parser.add_argument('--selneg', type=int, default=500,
-                            help="number of selected negative")
-
-
-class ParisM(Paris):
-    """ Metric Learning Edition of `Paris`
-    """
-
-    # selected classes for metric learning
-    SEL_LANDMARKS = Paris.ALL_LANDMARKS
-
-    def __init__(self, args):
-        Paris.__init__(self, args)
-
-        all_classes = ['background'] + Paris.ALL_LANDMARKS
+        init for metric learning
+        """
+        all_classes = ['background'] + cls.ALL_LANDMARKS
 
         pinfo("Preparing for metric learning ... ")
         self.trainlst = []
         qry_lbs = self.lbslst[self.qrylst]
         qry_mask = np.zeros(len(self.qrylst))
-        for sel_landmark in ParisM.SEL_LANDMARKS:
+        for sel_landmark in cls.SEL_LANDMARKS:
             classid = all_classes.index(sel_landmark)
             qry_mask[qry_lbs == classid] = 1
             candidates = [iid for iid in self.dblst[self.lbslst == (classid+1)]
@@ -506,6 +407,100 @@ class ParisM(Paris):
                       if iid not in self.trainlst]
         pinfo("Done\n")
 
+    @classmethod
+    def set_parser(cls, parser):
+        Builder.set_parser(parser)
+        parser.add_argument(
+            'img_dir', type=str,
+            help="dir containing images extracted from %s" % cls.IMG_DIR_SRC)
+        parser.add_argument(
+            'gt_dir', type=str,
+            help="dir containing groundtruth extracted from %s" % cls.GT_DIR_SRC)
+        parser.add_argument('--good', type=float, default=2.0,
+                            help="gain value for `good` results")
+        parser.add_argument('--ok', type=float, default=1.0,
+                            help="gain value for `ok` results")
+        parser.add_argument('--junk', type=float, default=0.0,
+                            help="gain value for `junk` results")
+        parser.add_argument('--selpos', type=int, default=cls.DEF_SELPOS,
+                            help="NO. of positive image for each class")
+        parser.add_argument('--selneg', type=int, default=cls.DEF_SELNEG,
+                            help="number of selected negative")
+
+
+class Oxford(VGGBuilder):
+    """ Oxford-dataset builder.
+    Download: http://www.robots.ox.ac.uk/~vgg/data/oxbuildings/
+
+    !!NOTE!! `all_souls_000183.jpg` appeares in both `all_souls` and
+             `radcliffe_camera` classes
+
+    ** Follow the original paper, queries are not excluded form the positive list.
+    """
+
+    ALL_LANDMARKS = ['all_souls', 'ashmolean', 'balliol', 'bodleian',
+                     'christ_church', 'cornmarket', 'hertford', 'keble',
+                     'magdalen', 'pitt_rivers', 'radcliffe_camera']
+    IMG_DIR_SRC = '`oxbuild_images.tgz`'
+    GT_DIR_SRC = '`gt_files_170407.tgz`'
+    DEF_SELPOS = 7
+    DEF_SELNEG = 500
+
+    def __init__(self, args):
+        """ Parse the Oxford dataset
+        """
+        VGGBuilder.__init__(self, args)
+
+
+class Paris(VGGBuilder):
+    """ Paris-dataset builder.
+    Download: http://www.robots.ox.ac.uk/~vgg/data/parisbuildings/
+
+    ** Follow the original paper, queries are not excluded form the positive list.
+    """
+
+    ALL_LANDMARKS = ['defense', 'eiffel', 'invalides', 'louvre', 'moulinrouge',
+                     'museedorsay', 'notredame', 'pantheon', 'pompidou',
+                     'sacrecoeur', 'triomphe']
+    IMG_DIR_SRC = '`paris_1.tgzk` and `paris_2.tgz`'
+    GT_DIR_SRC = '`paris_120310.tgz`'
+    DEF_SELPOS = 15
+    DEF_SELNEG = 500
+
+    def __init__(self, args):
+        """ Parse the Paris dataset
+        """
+        VGGBuilder.__init__(self, args)
+
+
+class OxfordM(Oxford):
+    """ Metric Learning Edition of `Oxford`
+    """
+
+    # selected classes for metric learning
+    SEL_LANDMARKS = ['all_souls', 'ashmolean', 'bodleian', 'christ_church',
+                     'hertford', 'magdalen', 'radcliffe_camera']
+
+    def __init__(self, args):
+        Oxford.__init__(self, args)
+        self.init_M()
+
+    @classmethod
+    def set_parser(cls, parser):
+        Oxford.set_parser(parser)
+
+
+class ParisM(Paris):
+    """ Metric Learning Edition of `Paris`
+    """
+
+    # selected classes for metric learning
+    SEL_LANDMARKS = Paris.ALL_LANDMARKS
+
+    def __init__(self, args):
+        Paris.__init__(self, args)
+        self.init_M()
+
     @classmethod
     def set_parser(cls, parser):
         Paris.set_parser(parser)
@@ -561,11 +556,11 @@ class Holidays(Builder):
         parser.add_argument('img_dir', type=str,
                             help="""dir containing images extracted from
                             `jpg1.tar.gz` and `jpg2.tar.gz`""")
-        parser.add_argument('--good', type=float, default=3.0,
+        parser.add_argument('--good', type=float, default=2.0,
                             help="gain value for `good` results")
-        parser.add_argument('--ok', type=float, default=2.0,
+        parser.add_argument('--ok', type=float, default=1.0,
                             help="gain value for `ok` results")
-        parser.add_argument('--junk', type=float, default=1.0,
+        parser.add_argument('--junk', type=float, default=0.0,
                             help="gain value for `junk` results")
 
 
@@ -611,88 +606,272 @@ class UKBench(Builder):
                             `ukbench.zip`""")
 
 
-class Caltech(Builder):
-    """ Caltech101/256-dataset builder.
-    Download: http://www.vision.caltech.edu/Image_Datasets/Caltech101/
-              http://www.vision.caltech.edu/Image_Datasets/Caltech256/
+class FolderBuilder(Builder):
+    """ This `Builder` is for the datasets with two-level folder structure,
+        where each sub-folder contains a class.
 
-    NOTE: the image list are generated by another program.
-        The list contains lines of image information:
-            `path_of_image_0` `class_label_of_image_0`
-            `path_of_image_1` `class_label_of_image_1`
-            `path_of_image_2` `class_label_of_image_2`
-            ...
+        256_ObjectCategories
+        ├── 001.ak47
+        │   ├── 001_0001.jpg
+        │   ├── ...
+        │   └── 001_0098.jpg
+        ├── ...
+        │
+        ├── 256.toad
+        │   ├── 256_0001.jpg
+        │   ├── ...
+        │   └── 256_0108.jpg
+        └── 257.clutter
+            ├── 257_0001.jpg
+            ├── ...
+            └── 257_0827.jpg
+
+      - The constructor will provide common process for preparing the dataset.
+      - For special needs on a dataset, extend this class and override the `pre_process`
+        and `post_process` rountines.
     """
 
-    def __init__(self, args, pos=1):
-        """
-        """
+    def __init__(self, args):
         Builder.__init__(self)
 
-        self.s2qlst = {pos: []}
+        self.pre_process(args)
 
-        pinfo("Loading image list ")
-        pinfo(".")
-        imglst = [line.split() for line in loadlist(args.imglst)]
-        pinfo(".")
-        self.imglst = [line[0] for line in imglst]
-        pinfo(".")
-        raw_lbs = [int(line[1]) for line in imglst]
-        uniqlbs = list(np.unique(raw_lbs))
-        self.lbslst = np.array([uniqlbs.index(lbl) for lbl in raw_lbs])
-        pinfo(".")
+        self.selcls = None
+        if args.selcls is not None:
+            with open(args.selcls) as lstf:
+                self.selcls = [line.strip() for line in lstf]
+
+        pinfo("Loading image list ...")
+        folders = sorted(os.listdir(args.img_dir));
+        self.classes = ["BACKGROUND"]
+        self.imglst = []
+        self.lbslst = []
+        v_discarded = []
+        # number of images per class
+        num_img_per_cls = args.trnum + args.vanum + args.tsnum
+
+        # scan the dataset
+        for fld in folders:
+            # if `selcls` is specified and cls_name is not in `it`,
+            # then skip this class
+            cls_name = self.get_cls_name(fld)
+            if self.selcls is not None and cls_name not in self.selcls:
+                continue
+
+            if fld == self.BG_CLASS:
+                label = 0
+                self.classes[0] = cls_name
+            else:
+                label = len(self.classes)
+                self.classes.append(cls_name)
+
+            # scan the subdir for images
+            sub_dir = os.path.join(args.img_dir, fld)
+            # get image names
+            imnames = []
+            for imname in os.listdir(sub_dir):
+                if imname.endswith('.jpg'):
+                    imnames.append(imname)
+                else:
+                    v_discarded.append(os.path.join(sub_dir, imname))
+            # shuffling the images
+            np.random.shuffle(imnames)
+            # randomly select `num_img_per_cls` images
+            imnames = sorted(imnames[:num_img_per_cls])
+            # append the selected images to self.imglst
+            self.imglst += [os.path.join(sub_dir, imname)
+                            for imname in imnames]
+            self.lbslst += [label for imname in imnames]
         pinfo(" Done!\n")
+        if len(v_discarded) > 0:
+            perr("** Discarded files:\n")
+            for disc in v_discarded:
+                perr("\t\t%s\n" % disc)
 
-        self.trainlst = np.ndarray(0, np.int64)
-        self.dblst = np.ndarray(0, np.int64)
-        self.qrylst = np.ndarray(0, np.int64)
 
-        imgnum = len(self.imglst)
-        clsnum = len(uniqlbs)
-        imgrange = np.array(range(imgnum))
-        for i in range(clsnum):
-            cur_ids = imgrange[self.lbslst == i]
-            np.random.shuffle(cur_ids)
-            self.trainlst = np.hstack((self.trainlst, cur_ids[:args.trnum]))
-            self.qrylst = np.hstack((self.qrylst,
-                                     cur_ids[args.trnum:
-                                             args.trnum + args.tsnum]))
-            self.dblst = np.hstack((self.dblst,
-                                    cur_ids[args.trnum:
-                                            args.trnum + args.tsnum]))
-
-        pinfo("\tGenerating label list ")
-        pinfo(".")
-        self.dlblst = self.lbslst[self.dblst]
-        pinfo(".")
-        self.qlblst = self.lbslst[self.qrylst]
-        pinfo(".")
-        pinfo(" Done!\n")
-
-        pinfo("\tGenerating pos list \n")
+        self.dblst = []
+        self.qrylst = []
         self.poslst = []
         self.amblst = []
-        qrynum = len(self.qrylst)
-        for qid in range(qrynum):
-            if qid % 10 == 0:
-                pinfo("\r\t\t%d/%d" % (qid, qrynum))
-            self.poslst.append(self.dblst[self.dlblst == self.qlblst[qid]])
-            self.amblst.append([])
-        self.s2qlst[pos] = self.poslst
-        pinfo("\r\t\t%d/%d\tDone!\n" % (len(self.qrylst), len(self.qrylst)))
+        self.trainlst = []
+        self.val_lst = []
+        self.s2qlst = {args.pos: []}
+        for i in range(len(self.selcls)):
+            # get training list
+            start_idx = num_img_per_cls * i
+            end_idx = start_idx + args.trnum
+            self.trainlst += range(start_idx, end_idx)
 
-    def __del__(self):
+            # get validation list
+            start_idx = end_idx
+            end_idx = start_idx + args.vanum
+            self.val_lst += range(start_idx, end_idx)
+
+            # get test list (query & db)
+            start_idx = end_idx
+            end_idx = start_idx + args.tsnum
+            self.qrylst += range(start_idx, end_idx)
+            self.dblst += range(start_idx, end_idx)
+
+            # get groundtruth
+            for j in range(args.tsnum):
+                self.poslst.append(range(start_idx, end_idx))
+                self.amblst.append([])
+
+        self.s2qlst[args.pos] = self.poslst
+
+        self.post_process(args)
+
+    def pre_process(self, args):
+        """
+        """
         pass
 
+    def post_process(self, args):
+        """
+        """
+        pass
+
+    def get_cls_name(self, folder):
+        """ Get class name from the folder name.
+        """
+        return folder
+
     @classmethod
     def set_parser(cls, parser):
         Builder.set_parser(parser)
-        parser.add_argument('imglst', type=str,
-                            help="image image list")
-        parser.add_argument('trnum', type=int, nargs='?', default=30,
-                            help="NO. of training images per class")
-        parser.add_argument('tsnum', type=int, nargs='?', default=20,
-                            help="NO. of test images per class")
+        parser.add_argument(
+            'img_dir', type=str,
+            help="root dir contains subfolders for each class")
+        parser.add_argument(
+            '--selcls', type=str, nargs='?', default=None,
+            help="list file contains name of selected classes")
+        parser.add_argument(
+            '--trnum', type=int, nargs='?', default=cls.DEF_TRNUM,
+            help="NO. of training images per class")
+        parser.add_argument(
+            '--vanum', type=int, nargs='?', default=cls.DEF_VANUM,
+            help="NO. of validation images per class")
+        parser.add_argument(
+            '--tsnum', type=int, nargs='?', default=cls.DEF_TSNUM,
+            help="NO. of test images per class")
+        parser.add_argument(
+            '--pos', type=float, default=1.0,
+            help="gain value for `pos` results")
+
+
+class Caltech101(FolderBuilder):
+    """ Caltech101-dataset builder.
+    Download: http://www.vision.caltech.edu/Image_Datasets/Caltech101/
+    """
+
+    DEF_TRNUM = 30
+    DEF_VANUM = 0
+    DEF_TSNUM = 20
+    BG_CLASS = "BACKGROUND_Google"
+
+    def __init__(self, args):
+        FolderBuilder.__init__(self, args)
+
+
+class Caltech256(FolderBuilder):
+    """ Caltech256-dataset builder.
+    Download: http://www.vision.caltech.edu/Image_Datasets/Caltech256/
+    """
+
+    DEF_TRNUM = 40
+    DEF_VANUM = 0
+    DEF_TSNUM = 25
+    BG_CLASS = "257.clutter"
+
+    def __init__(self, args):
+        FolderBuilder.__init__(self, args)
+
+    def get_cls_name(self, folder):
+        return folder.split(".")[1]
+
+    def pre_process(self, args):
+        pass
+
+    def post_process(self, args):
+        pass
+
+
+# class Caltech(Builder):
+#     """ Caltech101/256-dataset builder.
+#     Download: http://www.vision.caltech.edu/Image_Datasets/Caltech101/
+#               http://www.vision.caltech.edu/Image_Datasets/Caltech256/
+# 
+#     NOTE: the image list are generated by another program.
+#         The list contains lines of image information:
+#             `path_of_image_0` `class_label_of_image_0`
+#             `path_of_image_1` `class_label_of_image_1`
+#             `path_of_image_2` `class_label_of_image_2`
+#             ...
+#     """
+# 
+#     def __init__(self, args, pos=1.0):
+#         """
+#         """
+#         Builder.__init__(self)
+# 
+#         self.s2qlst = {pos: []}
+# 
+#         pinfo("Loading image list ...")
+#         imglst = [line.split() for line in loadlist(args.imglst)]
+#         self.imglst = [line[0] for line in imglst]
+#         raw_lbs = [int(line[1]) for line in imglst]
+#         uniqlbs = list(np.unique(raw_lbs))
+#         self.lbslst = np.array([uniqlbs.index(lbl) for lbl in raw_lbs])
+#         pinfo(" Done!\n")
+# 
+#         self.trainlst = np.ndarray(0, np.int)
+#         self.dblst = np.ndarray(0, np.int)
+#         self.qrylst = np.ndarray(0, np.int)
+# 
+#         imgnum = len(self.imglst)
+#         clsnum = len(uniqlbs)
+#         imgrange = np.array(range(imgnum))
+#         for i in range(clsnum):
+#             cur_ids = imgrange[self.lbslst == i]
+#             np.random.shuffle(cur_ids)
+#             self.trainlst = np.hstack((self.trainlst, cur_ids[:args.trnum]))
+#             self.qrylst = np.hstack((self.qrylst,
+#                                      cur_ids[args.trnum:
+#                                              args.trnum + args.tsnum]))
+#             self.dblst = np.hstack((self.dblst,
+#                                     cur_ids[args.trnum:
+#                                             args.trnum + args.tsnum]))
+# 
+#         pinfo("\tGenerating label list ...")
+#         self.dlblst = self.lbslst[self.dblst]
+#         self.qlblst = self.lbslst[self.qrylst]
+#         pinfo(" Done!\n")
+# 
+#         pinfo("\tGenerating pos list \n")
+#         self.poslst = []
+#         self.amblst = []
+#         qrynum = len(self.qrylst)
+#         for qid in range(qrynum):
+#             if qid % 10 == 0:
+#                 pinfo("\r\t\t%d/%d" % (qid, qrynum))
+#             self.poslst.append(self.dblst[self.dlblst == self.qlblst[qid]])
+#             self.amblst.append([])
+#         self.s2qlst[pos] = self.poslst
+#         pinfo("\r\t\t%d/%d\tDone!\n" % (len(self.qrylst), len(self.qrylst)))
+# 
+#     @classmethod
+#     def set_parser(cls, parser):
+#         Builder.set_parser(parser)
+#         parser.add_argument(
+#             'img_dir', type=str,
+#             help="`256_ObjectCategories` extracted from `256_ObjectCategories.tar`")
+#         parser.add_argument('imglst', type=str,
+#                             help="image image list")
+#         parser.add_argument('trnum', type=int, nargs='?', default=40,
+#                             help="NO. of training images per class")
+#         parser.add_argument('tsnum', type=int, nargs='?', default=25,
+#                             help="NO. of test images per class")
 
 
 class ImageNet2012(Builder):
@@ -707,7 +886,7 @@ class ImageNet2012(Builder):
             ...
     """
 
-    def __init__(self, args, pos=1):
+    def __init__(self, args, pos=1.0):
         """
         """
         Builder.__init__(self)
@@ -764,12 +943,13 @@ class ImageNet2012(Builder):
                             help="""validation image list""")
 
 
-DATASETS = {
-    'caltech':     Caltech,
+DS_BUILDERS = {
+#     'caltech':     Caltech,
+    'cal256':      Caltech256,
     'imgnet2012':  ImageNet2012,
     'ukbench':     UKBench,
     'holidays':    Holidays,
     'parism':      ParisM,
     'paris':       Paris,
     'oxfordm':     OxfordM,
-    'oxford':      Oxford}
+    'oxford':      Oxford}
\ No newline at end of file
